function scan_data::match_detector_name, str, strict=strict, show_error=show_error
;+
; NAME:               scan_data::match_detector_name
;
; PURPOSE:            look up detector group (ie, summed detectors) 
;                     by name (or closest match), return array index
;
; CATEGORY:           scan_data class lib
;
; CALLING SEQUENCE:   n = scan_data->match_detector_name(string)
;
; INPUTS:             str        - string to look up as 'detector group name'
;
; KEYWORD PARAMETERS: strict     - flag to force exact string match
;                     show_error - flag to show error messagee on failure
;
; OUTPUTS:            integer index in scan_data.sums for detector group
;
; PROCEDURE:          the input string is first checked for an exact match with
;                     the set of detector names in scan_data.det_names.  if no
;                     exact match is found, then the passed string is checked 
;                     for exact match of the first word (blank delimited) of 
;                     each name in scan_data.det_names.  finally, if strict=0 
;                     (the default), the first close match is chosen.
;
;
; EXAMPLE:            d = obj_new('scan_data','scan_file_001.dat')
;                     n = d->match_detector_name('Cu')
;
; MODIFICATION HISTORY:  2001-aug-05  m newville
;
;-

if (n_elements(str) le 0) then begin
    print, " match_detector_name error -- no string given"
    return, -1
endif
s  = strlowcase(str)
n =  n_elements(*self.det_names) 
; look for exact match
for i = 0, n - 1 do begin
    if (s eq strlowcase((*self.det_names)[i])) then return, i
endfor

; inexact match:  check first word
for i = 0, n - 1 do begin
    sx   = str_sep(strlowcase((*self.det_names)[i]), ' ')
    if (s eq sx[0]) then return, i
endfor
;
; last resort: check for any similarity
if (keyword_set(strict) eq 0) then begin
    for i = 0, n - 1 do begin
        j = strpos(strlowcase((*self.det_names)[i]), s)
        if (j ge 0) then begin
            print, s,  ' is sort of like ', (*self.det_names)[i]
            return, i
        endif
    endfor
endif

if (keyword_set(show_error) ne 0) then begin
    print, " can't find  detector named ", str
    print, " available detectors: "
    self->show_detectors
endif
return, -1
end


function scan_data::get_map, name=name, index=index, $
             use_sum=use_sum, use_raw=use_raw, $
             norm=norm, inorm=inorm

;+
; NAME:               scan_data::get_map
;
; PURPOSE:            return map data from scan_data object
;
; CATEGORY:           scan_data class lib
;
; CALLING SEQUENCE:   map = scan_data->get_map(name=name, index=index,...)
;
; INPUTS:             name       - string to look up as 'detector group name'
;
; KEYWORD PARAMETERS: name       - flag to force exact string match
;                     index      - flag to show error messagee on failure
;
; OUTPUTS:            2d data for requested map
;
; PROCEDURE:          the ...
;
;
; EXAMPLE:            d   = obj_new('scan_data','scan_file_001.dat')
;                     map = d->get_map(name='Cu')
;
; MODIFICATION HISTORY:  2001-aug-05  m newville
;
;-

stitle = ''
uname  = ''
nname  = ''
uindex = -1
nindex = -1
usum   =  1
do_norm=  0
empty  = fltarr(2,2) - 99
if (n_elements(name)    ne 0)  then uname = name
if (n_elements(index)   ne 0)  then uindex= index
if (n_elements(use_sum) ne 0)  then usum  = 1
if (n_elements(use_raw) ne 0)  then usum  = 0
if (n_elements(norm)    ne 0)  then nname = norm
if (n_elements(inorm)   ne 0)  then nindex=inorm

if (nname ne '') then do_norm = 1
if (nindex ne -1) then do_norm = 2
; 
;
if (self.dimension eq 1) then begin
    print, ' get_map:  not a map file -- try get_data '
    return, empty
endif
if ((uname eq '') and (uindex eq -1)) then begin
    print, ' get_map:  no element index or name given'
    return, empty
endif else if (uname ne '') then begin
    uindex = self->match_detector_name(uname,/show_error)
    if (uindex lt 0) then return, empty
endif

if (usum eq 0) then begin
    map = (*self.raw)[*,*,uindex]
    stitle = 'Raw Detector ' + strtrim(uindex,2)
endif else if (usum eq 1) then begin
    map = (*self.sums)[*,*,uindex]
    stitle = (*self.det_names)[uindex]
endif
;
; determine normalization array
if (do_norm eq 1) then begin
    nindex = self->match_detector_name(nname, /show_error)
    if (nindex lt 0) then return, empty
endif
;
; normalize here
if ((nindex ge 0) and (do_norm ge 1)) then begin
    if (usum eq 0) then begin
        map = map / (*self.raw)[*,*,nindex]
        stitle = stitle + ' / Raw Detector ' +  strtrim(nindex,2)
    endif else if (usum eq 1) then begin
        map = map / (*self.sums)[*,*,nindex]
        stitle = stitle + ' / ' +  (*self.det_names)[nindex]
    endif

endif
self.subtitle = stitle
return, map
end


;;
pro scan_data::show_map, name=name,  index=index, $
             use_sum=use_sum, use_raw=use_raw, $
             norm=norm, inorm=inorm
;+
; NAME:               scan_data::show_map
;
; PURPOSE:            plot map data from scan_data object using image_display
;
; CATEGORY:           scan_data class lib
;
; CALLING SEQUENCE:   scan_data->show_map, name=name, index=index, ...
;
; INPUTS:             name       - string to look up as 'detector group name'
;
; KEYWORD PARAMETERS: name       - flag to force exact string match
;                     index      - flag to show error messagee on failure
;
; OUTPUTS:            2d data for requested map
;
; PROCEDURE:          the ...
;
;
; EXAMPLE:            d   = obj_new('scan_data','scan_file_001.dat')
;                     map = d->get_map(name='Cu')
;
; MODIFICATION HISTORY:  2001-aug-05  m newville
;
;-

map = self->get_map(name=name, index=index, use_sum=use_sum, $
                    use_raw=use_raw, norm=norm, inorm=inorm)

if ((map(0,0) ne -99)  or (map(1,0) ne -99) or (n_elements(map) gt 4)) then begin
    image_display, map, xdist=(*self.x),ydist=(*self.y),$
      title=self.filename, subtitle=self.subtitle
endif

self.subtitle=''
return
end

;;
pro scan_data::show_correl, x=x, y=y, _extra=extra

;+
; NAME:               scan_data::show_correl
;
; PURPOSE:            plot map data from scan_data object using image_display
;
; CATEGORY:           scan_data class lib
;
; CALLING SEQUENCE:   scan_data->show_map, name=name, index=index, ...
;
; INPUTS:             name       - string to look up as 'detector group name'
;
; KEYWORD PARAMETERS: name       - flag to force exact string match
;                     index      - flag to show error messagee on failure
;
; OUTPUTS:            2d data for requested map
;
; PROCEDURE:          the ...
;
;
; EXAMPLE:            d   = obj_new('scan_data','scan_file_001.dat')
;                     map = d->get_map(name='Cu')
;
; MODIFICATION HISTORY:  2001-aug-05  m newville
;
;-

map1 = self->get_map(name=x, /use_sum)
lab1 = self.subtitle

map2 = self->get_map(name=y, /use_sum)
lab2 = self.subtitle

self.subtitle=''
if ((map1(0,0) ne -99)  or (map1(1,0) ne -99) or (n_elements(map1) gt 4)) then begin
    plot, map1, map2, psym=1, xtitle=lab1, ytitle=lab2, title=self.filename, $
      back=set_color('white'), color=set_color('black'), _extra=extra
endif

return
end

;;

function scan_data::get_orig_detector_list, name=name, index=index
;
; return list of detectors in the original 'raw' map for sum of a particular named group
;
uname = ''
uindex= -1
if (n_elements(name)    ne 0)  then uname = name
if (n_elements(index)   ne 0)  then uindex= index

if (uindex lt 0) then begin
    uindex = self->match_detector_name(uname, /show_error)
endif

if (uindex lt 0) then return, uindex

n   = n_elements((*self.det_orig)) 
tmp = intarr(n) - 1
cnt = 0
for i = 0, n - 1 do begin
    if ((*self.det_orig)[i] eq uindex) then begin
        tmp[cnt] = i
        cnt = cnt + 1
    endif
endfor
return, tmp(0:cnt-1)
end

function scan_data::get_detector_list, name=name, index=index
;
; return list of detectors in 'raw' map for sum of a particular named group
;
uname = ''
uindex= -1
if (n_elements(name)    ne 0)  then uname = name
if (n_elements(index)   ne 0)  then uindex= index

if (uindex lt 0) then begin
    uindex = self->match_detector_name(uname, /show_error)
endif

if (uindex lt 0) then return, uindex

n   = n_elements((*self.det_list)) 
tmp = intarr(n) - 1
cnt = 0
for i = 0, n - 1 do begin
    if ((*self.det_list)[i] eq uindex) then begin
        tmp[cnt] = i
        cnt = cnt + 1
    endif
endfor
return, tmp(0:cnt-1)
end

function scan_data::set_detector_list, name=name, index=index, list=list
;
; return list of detectors in 'raw' map for sum of a particular named group
;
uname = ''
uindex= -1
if (n_elements(name)    ne 0)  then uname = name
if (n_elements(index)   ne 0)  then uindex= index

if (uindex lt 0) then begin
    uindex = self->match_detector_name(uname, /show_error)
endif

if (uindex lt 0) then begin
    print,  ' set_detector_list : no detector to set'
    return, -1
endif else if (n_elements(list)   le 0)  then begin
    print,  ' set_detector_list : no detector list given.'
endif

n     = n_elements((*self.det_list)) 
(*self.sums)[*,*,uindex] = 0
for i = 0, n_elements(list) - 1 do begin
    j = list[i]
    (*self.det_list)[j] = uindex
    (*self.sums)[*,*,uindex]  = (*self.sums)[*,*,uindex]  + (*self.raw)[*,*,j] 
endfor

return, list
end


pro scan_data::map_all_detectors, name=name, index=index, _extra=extra
self->plot_all_detectors, name=name, index=index, _extra=extra
return
end

pro scan_data::plot_all_detectors, name=name, index=index, _extra=extra
;
;  plot all detectors in a named group

if (n_elements(name) ne 0) then begin
    n  = self->match_detector_name(name)
    if (n ge 0) then   name = (*self.det_names)[n]
endif
list = self->get_detector_list(name=name, index=index)
ytitle = 'detectors ' + strtrim(list[0],2) + ':' +  strtrim(list[n_elements(list)-1],2) 
if (n_elements(name) ne 0) then ytitle = name + ' ( ' + ytitle + ' )'
dim    = self.dimension


if (n_elements(list) ge 1) then begin
    if (dim eq 1) then begin
        ymax = max( (*self.raw)[*,list] )
        ymin = min( (*self.raw)[*,list] )
        self->plot,  index=list[0], /use_raw, $
          yrange=[ymin,ymax], ytitle=ytitle, _extra=extra
        for i = 1, n_elements(list) - 1 do begin
            self->oplot, index=list[i], /use_raw, _extra=extra
        endfor
    endif else begin
        for i = 0, n_elements(list) - 1 do begin
            self->show_map,  index=list[i], /use_raw, _extra=extra
        endfor
    endelse
endif

return
end




function scan_data::get_data, name=name, index=index, $
             use_sum=use_sum, use_raw=use_raw, $
             norm=norm, inorm=inorm

empty  = fltarr(2)
data   = empty
stitle = ''
uname  = ''
nname  = ''
uindex = -1
nindex = -1
usum   =  1
do_norm=  0
is_map =  0
empty  = fltarr(2,2) - 99
if (n_elements(name)    ne 0)  then uname = name
if (n_elements(index)   ne 0)  then uindex= index
if (n_elements(use_sum) ne 0)  then usum  = 1
if (n_elements(use_raw) ne 0)  then usum  = 0
if (n_elements(norm)    ne 0)  then nname = norm
if (n_elements(inorm)   ne 0)  then nindex=inorm

if (nname ne '') then do_norm = 1
if (nindex ne -1) then do_norm = 2
; 
;
if (self.dimension eq 2) then begin
    print, ' get_data:  this is a map file -- getting row 0 (try get_map)!'
    is_map = 1
endif
if ((uname eq '') and (uindex eq -1)) then begin
    print, ' get_data:  no element index or name given'
    return, empty
endif else if (uname ne '') then begin
    uindex = self->match_detector_name(uname,/show_error)
    if (uindex lt 0) then return, empty
endif

if (usum eq 0) then begin
    if (is_map eq 1) then begin
        data = (*self.raw)[*,0,uindex]
    endif else begin
        data = (*self.raw)[*,uindex]
    endelse
    stitle = 'Raw Detector ' + strtrim(uindex,2)
endif else if (usum eq 1) then begin
    if (is_map eq 1) then begin
        data = (*self.sums)[*,0,uindex]
    endif else begin
        data = (*self.sums)[*,uindex]
    endelse
    stitle = (*self.det_names)[uindex]
endif
;
; determine normalization array
if (do_norm eq 1) then begin
    nindex = self->match_detector_name(nname, /show_error)
    if (nindex lt 0) then return, empty
endif
;
; normalize here
if ((nindex ge 0) and (do_norm ge 1)) then begin
    if (usum eq 0) then begin
        if (is_map eq 1) then begin
            data = data / (*self.raw)[*,0,nindex]
        endif else begin
            data = data / (*self.raw)[*,nindex]
        endelse
        stitle = stitle + ' / Raw Detector ' +  strtrim(nindex,2)
    endif else if (usum eq 1) then begin
        if (is_map eq 1) then begin
            data = data / (*self.sums)[*,0,nindex]
        endif else begin
            data = data / (*self.sums)[*,nindex]
        endelse
        stitle = stitle + ' / ' +  (*self.det_names)[nindex]
    endif
endif
self.subtitle = stitle
return, data
end


;;
pro scan_data::plot, name=name, index=index, $
             use_sum=use_sum, use_raw=use_raw, $
             norm=norm, inorm=inorm, color=color, bw=bw,$
             _extra=extra

ydat  = self->get_data(name=name,  index=index, $
                       use_sum=use_sum, use_raw=use_raw, $
                       norm=norm, inorm=inorm)

lab1 = self.subtitle
self.subtitle=''

ucol = set_color((*self.plot_colors)[0])
if (n_elements(color) ne 0) then ucol = set_color(color)
if (keyword_set(bw) ne 0)   then ucol = set_color('black')

plot, (*self.x),  ydat,  xtitle=self.xpos, ytitle=lab1, title=self.filename, chars=1.5, thick=2, $
  back=set_color('white'), color=set_color('black'), _extra=extra,  /nodata
oplot, (*self.x),  ydat, psym=0, color=ucol, _extra=extra
self.nplot = 0
return
end

pro scan_data::oplot, name=name, index=index, $
             use_sum=use_sum, use_raw=use_raw, $
             norm=norm, inorm=inorm, color=color, bw=bw, $
             _extra=extra

ydat  = self->get_data(name=name,  index=index, $
                       use_sum=use_sum, use_raw=use_raw, $
                       norm=norm, inorm=inorm)
self.nplot = self.nplot + 1
lab1 = self.subtitle
self.subtitle=''
nc   = n_elements((*self.plot_colors))
ucol = set_color((*self.plot_colors)[self.nplot mod nc])
if (keyword_set(bw) ne 0) then begin
    ucol = set_color('black')
    nc   = 1
endif
usym =-self.nplot/nc

if (n_elements(color) ne 0) then ucol = set_color(color)
oplot, (*self.x),  ydat, psym=usym, color=ucol, _extra=extra

return
end

;;
function scan_data::read_data_file, file
; read epics scan data file
@scan_dims

M_GROUPS    = 50
MDIM        = 500
retval = -1

;
; read detector and positioner arrays from ascii-dump versions 
; of data-catcher files

if ((n_elements(file) eq 0) or  (keyword_set(help) ne 0)) then begin
    print, ' function scan_data::read_data_file(filename)'
    return, retval
endif
;

str     = '; '
print, format='(3a,$)', 'opening file ', file, ' ... '
openr, lun, file, /get_lun
tmp_x      = fltarr(MDIM, MAX_POS)
tmp_det    = fltarr(MDIM, MDIM, MAX_DET)
vars       = fltarr(MAX_POS+MAX_DET)
tmp_y      = fltarr(MDIM)
det_name   = strarr(M_GROUPS)
pos_name   = strarr(MAX_POS)
ypos       = ''
ypv        = ''
det_list   = intarr(MAX_DET)
group_list = strarr(M_GROUPS)
Det_String = strarr(m_groups)

npts       = -1
nrow       = -1
ngroups    = -1
npos       = 0
ndet       = 0
first_line = 1
ncols      = 1
nline      = 1
read_labels= 1
dimen      = 2
do_parsing = 0
print, 'reading ...'
while not (eof(lun)) do begin
    readf,lun,str
    nline = nline + 1
    string = strtrim(str,2)
    if (strlen(string) gt 1) then begin
        char1 = strmid(string, 0, 1) 
        if ((char1 ne ';') and (read_labels eq 0)) then begin
; read data
            npts = npts + 1
            reads, string, vars            
            if (nrow lt 0) then nrow = 0
            if (nrow eq 0) then  tmp_x[npts, 0:npos-1] = vars[0:npos-1]
            tmp_det[npts, nrow, 0:ndet-1] = vars[npos:npos+ndet-1]
        endif else begin
            if (first_line eq 1) then begin
                stmp = strmid(string, 0, 12)
                s2   = strmid(string, 12, 13)
                sx   = str_sep(strtrim(s2,2), ' ')
                if (stmp ne '; Epics Scan') then begin
                    print, ' Not a valid scan file! '
                    goto, endread
                endif
                dimen = fix(sx[0])
                first_line = 0
            endif
            char1 = strmid(string, 0, 1) 
            char3 = strmid(string, 0, 3) 
            char8 = strmid(strtrim(string,2) , 0,8) 
            if ((dimen ge 2) and (char3 eq ';2D'))  then begin
                nrow = nrow + 1
                if ((nrow gt 5) and ( ((nrow) mod 10) eq 0)) then print, ";"
                print, format='(a,i3,$)', '  ' , nrow
                sc = strmid(string,3, strlen(string))
                sx = str_sep(strtrim(sc,2), ' ' )
                tmp_y[nrow] = sx[1]
                npts_old = npts
                npts = -1
                if (ypv eq '') then begin
                    n = strpos(string,':', /reverse_search)
                    ypv = strmid(string,4, n-4)
                endif
            endif else if (char8 eq ';=======')  then begin
; signal to start looking at 
                do_parsing = 1
            endif else if ((read_labels eq 1) and (char8 eq ';-------'))  then begin
; read column label: done reading list of pos/det, so  declare size of vars here
                read_labels = 0
                readf,lun,string
                vars = fltarr(npos+ndet)
            endif else if ( do_parsing eq 1)  then begin
; reading list of positioner/detectors for 1d scan
                if (char3 eq '; P') then begin
                    npos = npos + 1
                    n  = -1 + fix(strmid(string,3,3))
                    i1 = strpos(string, '{')
                    i2 = strpos(string, '}')
                    pos_name[n]=strmid(string, i1+1, i2-i1-1)  
                endif else if (char3 eq '; D') then begin
                    ndet = ndet + 1
                    slen   = strlen(string) - 1
                    clast  = strmid(string, slen, slen)
                    if (clast eq 'N') then begin
                        roi = strmid(string, slen-2, slen-1)
                    endif else begin
                        roi = strmid(string, slen-1, slen)
                    endelse
                    ds  = strmid(string, 3, 2)
; force all non-mca detectors (like, scalars) into individual groups
                    if (strpos(string,'mca') lt 2) then  roi = string
                    for i = 0, m_groups - 1 do begin
                        if det_string[i] eq '' then begin
                            ngroups      = i
                            det_string[i]= roi
                            group_list[i]= ds
                            i1  = strpos(string, '{')
                            i2  = strpos(string, '}')
                            det_name[i]=strmid(string, i1+1, i2-i1-1)  
                            i1  = strpos(det_name[i], 'mca')
                            if (i1 ge 0) then begin
                                i2  = strpos(det_name[i], ':')
                                il  = i1+3 > i2
                                det_name[i] = strmid(det_name[i],il+1,strlen(det_name[i]))
                            endif
                            goto, det_found
                        endif else if(det_string[i] eq roi) then begin
                            group_list[i] = group_list[i] + ' ' + ds
                            goto, det_found
                        endif
                    endfor
                endif
            endif  else if ((ypos eq '') and (ypv ne '')) then begin
;
; searching for name of 2D positioner  in list of saved values
;
                i1  = strpos(string, '(')
                i2  = strpos(string, ')')
                if (i2 gt i1) then begin
                    pv =strmid(string, i1+1, i2-i1-1)  
                    if (pv eq ypv) then ypos = strtrim(strmid(string,2,i1-2),2)
                endif
            endif
        endelse
det_found:
    endif
endwhile

print, " Available groups from map: ", ngroups
dname    = strarr(ngroups+1)
detectors= intarr(MAX_DET) - 1
for i = 0, ngroups do begin
    if det_name[i] ne ''  then begin
        dname[i] = strtrim(det_name[i],2)
        g        = strtrim(group_list[i],2)
        length   = string_array(g, det_list)
        for j = 0, length-1 do  detectors[det_list[j]-1] = i
    endif
endfor


; print, ' detector list:'
; for i = 0, MAX_DET -1 do print, i, detectors[i]

nx   = npts
if (nx eq -1) then nx = npts_old
ny   = nrow
; print, " nx,ny = ", nx, ny, npts , npts_old
y    = fltarr(ny+1)
da   = fltarr(nx+1, ny+1, ndet)
x    = fltarr(nx+1)
for i = 0, nx do begin
    for j = 0, ny do begin
        for k = 0, ndet-1 do da(i,j,k) = tmp_det(i,j,k)
   endfor
endfor

for j = 0, ny do  y[j] = tmp_y[j] 
for j = 0, nx do  x[j] = tmp_x[j,0] 
;;    for k = 0, npos-1 do x(i,k) = tmp_x(i,k)

sums     = fltarr(nx+1,ny+1,ngroups+1)
for i = 0, ngroups do begin
    for j = 0, MAX_DET - 1 do begin
        if (detectors[j] eq i) then  sums(*,*,i) = sums(*,*,i) + da(*,*,j)
    endfor
endfor
;
; refom 1d data
if (dimen eq 1) then begin
    da  = reform(da)
    sums= reform(sums)
endif

retval  = 0

endread:

close, lun
free_lun, lun

self.dimension = dimen
self.filename  = file
self.xpos      = pos_name[0]
self.ypos      = ypos
self.raw       = ptr_new(da)
self.sums      = ptr_new(sums)
self.x         = ptr_new(x)
self.y         = ptr_new(y)
self.det_names = ptr_new(dname)
self.det_list  = ptr_new(detectors)
self.det_orig  = ptr_new(detectors)

return, retval
end


pro scan_data::set_param, par, val
;
; returns a single attribute of a particular scan
;
is = 0
if (keyword_set(iscan) ne 0) then is = iscan
if (keyword_set(par) ne 0) then begin
    case par of
        'dimension':    self.dimension    = val
        'filename':     self.filename     = val
        'raw_data':     (*self.raw)       = val
        'sums':         (*self.sums)      = val
        'x':            (*self.x)         = val
        'y':            (*self.y)         = val
        'xpos':         self.xpos         = val
        'ypos':         self.ypos         = val
        'det_names':    (*self.det_names) = val
        'det_list':     (*self.det_list)  = val
        'plot_colors':  (*self.plot_colors)  = val
    endcase
endif
return
end

function scan_data::get_x
return, self->get_param('x')
end

function scan_data::get_y
return, self->get_param('y')
end

function scan_data::get_param, par
;
; return a copy of an object member structure
; for outside manipulation and later 'set_param'ing
val = 0
if (keyword_set(par) ne 0) then begin
    case par of
        'dimension':    val = self.dimension
        'filename':     val = self.filename
        'raw_data':     val = (*self.raw)
        'sums':         val = (*self.sums)
        'x':            val = (*self.x)
        'y':            val = (*self.y)
        'xpos':         val = self.xpos
        'ypos':         val = self.ypos
        'det_names':    val = (*self.det_names)
        'det_list':     val = (*self.det_list)
        'det_orig':     val = (*self.det_orig)
        'plot_colors':  val = (*self.plot_colors)
    endcase
endif

return, val
end


pro scan_data::help
;+
; NAME:
;
;
;
; PURPOSE:
;
;
;
; CATEGORY:
;
;
;
; CALLING SEQUENCE:
;
;
;
; INPUTS:
;
;
;
; OPTIONAL INPUTS:
;
;
;
; KEYWORD PARAMETERS:
;
;
;
; OUTPUTS:
;
;
;
; OPTIONAL OUTPUTS:
;
;
;
; COMMON BLOCKS:
;
;
;
; SIDE EFFECTS:
;
;
;
; RESTRICTIONS:
;
;
;
; PROCEDURE:
;
;
;
; EXAMPLE:
;
;
;
; MODIFICATION HISTORY:
;
;-

print, ' Methods for scan_data class:'
print, ' --------------------------------------------------------'
print, ' function get_map '
print, ' pro      show_map '
print, ' pro      show_correl '
print, ' pro      show_detectors'
print, ' function read_data_file'
print, ' pro      help'
print, ' pro      set_param'
print, ' function get_param'
print, ' pro      show_params'
print, ' function match_detector_name'
print, ' '
self->show_params

return
end

pro scan_data::show_params
;
; list available parameters
;
print, ' Parameters accessible for scan file '
print, '   using get_param / set_param:'
print, ' --------------------------------------------------------'
print, '   name                meaning'
print, ' filename        scan file name '
print, ' dimension       scan dimension'
print, ' raw_data        complete set of raw detector data  '
print, ' sums            summed detectors, grouped by detector element '
print, ' det_names       names for summed detector groups'
print, ' det_list        2d integer array list of raw detectors for sums'
print, ' x               array of x positions '
print, ' y               array of y positions (for 2d scans)'
print, ' xpos            name of x positioner'
print, ' ypos            name of y positioner (for 2d scans)'
print, ' --------------------------------------------------------'
return
end


pro scan_data::show_detectors
n =  n_elements(*self.det_names) 

print, format='(a,$)', '[  '
for i = 0, n - 2 do begin
    print, format='(a,a,$)', (*self.det_names)[i], ', '
endfor
print, format='(a,a)', (*self.det_names)[n-1], ' ]'
return
end


function scan_data::init, file=file
;

cols = ['blue', 'red', 'black', 'magenta', 'forestgreen']
self.plot_colors =   ptr_new(cols)


if (n_elements(file) ne 0) then begin
    u = self->read_data_file(file)
    if (u eq 1) then begin
        print, ' file ', file , ' is not a valid scan data file.'
    endif else begin
        print, '   OK.'
    endelse
endif
return, 1
end

pro scan_data__define
;+
; NAME:
;       EPICS_SCAN__DEFINE
;
; PURPOSE:
;       Defines an object for EPICS scan data files from epics_scan object
;
; CATEGORY:
;       EPICS scan class library.
;
; CALLING SEQUENCE:
;       dat = obj_new('scan_data')
;
;
; INPUTS:
;       None.
;
; OPTIONAL INPUTS:
;       None.
;
; KEYWORD PARAMETERS:
;       None.
;
;
; OUTPUTS:
;       Return value will contain the object reference.
;
;
; OPTIONAL OUTPUTS:
;       None.
;
;
; COMMON BLOCKS:
;       None.
;
;
; SIDE EFFECTS:
;       EPICS_Scan object is created.
;
;
; RESTRICTIONS:
;       This routine is not called directly, but by IDL's own obj_new()
;
; PROCEDURE:
;
;
; EXAMPLE:
;       data = obj_new('scan_data', 'my_data_file.001')
;       data->read_scan_file('my_data_file.001')
;
; MODIFICATION HISTORY:
;       Aug 03 2001: M Newville
;-
@scan_dims

MAX_SUM_ELEMS = 16  ; max number of detectors to add together for sum

p = ptr_new()
scan_data = { scan_data, filename:   ' ', dimension: 1, $
              subtitle:' ', $
              raw:  p, sums:p,  plot_colors:p, nplot:0, $
              x: p, y: p,  xpos: '', ypos:'', $
              det_names: p, det_list: p, det_orig: p} 

end



